@@ -16,9 +16,10 @@ from .utils.compat import weakref
 import numpy as np
 
 # LOCAL
-from . import great_circle_arc
+from . import great_circle_arc as gca
 from . import math_util
 from . import vector
+from . import polygon as mpolygon
 
 __all__ = ['Graph']
 
@@ -83,8 +84,7 @@ class Graph:
                 empirical test cases. Relative threshold based on
                 the actual sizes of polygons is not implemented.
             """
-            return great_circle_arc.same_point(self._point, other._point,
-                                               tol=thresh)
+            return np.array_equal(self._point, other._point)
 
 
     class Edge:
@@ -488,7 +488,14 @@ class Graph:
         self._sanity_check("union - remove 3ary edges")
         self._remove_orphaned_nodes()
         self._sanity_check("union - remove orphan nodes", True)
-        return self._trace()
+
+        poly = self._trace()
+        for source_poly in self._source_polygons:
+            inside_point = source_poly.inside
+            break
+        else:
+            inside_point = None
+        return mpolygon.SphericalPolygon((poly, ), inside=inside_point)
 
     def intersection(self):
         """
@@ -509,14 +516,22 @@ class Graph:
         self._sanity_check("intersection - remove cut lines")
         self._remove_orphaned_nodes()
         self._sanity_check("intersection - remove orphan nodes", True)
-        return self._trace()
+
+        poly = self._trace()
+        if not self._contains_inside_point(poly):
+            poly = poly.invert_polygon()
+        return poly
 
     def disjoint_polygons(self):
         """
-        Convert a graph containing cut lines into a list of disjoint polygons
+        Convert a graph containing cut lines and self intersections
+        into a list of disjoint polygons
         """
-        if self._remove_cut_lines():
-            self._sanity_check("disjoint - remove cut lines")
+        changed = self._remove_cut_lines()
+        self._sanity_check("disjoint - remove cut lines")
+        changed = self._find_all_intersections() or changed
+        self._sanity_check("disjoint - find all intersections")
+        if changed:
             polygons = self._trace_polygons()
         else:
             polygons = list(self._source_polygons)
@@ -589,21 +604,23 @@ class Graph:
         edges = sorted(self._edges, key=edge_order)
         starts, ends = self._get_edge_points(edges)
 
-        nodes = sorted(self._nodes, key=node_order) ## DBG FIX
+        nodes = sorted(self._nodes, key=node_order)
         nodes_array = np.array([x._point for x in nodes])
 
         # Split all edges by any nodes that intersect them
+        changed = False
         while len(edges) > 1:
             AB = edges.pop(0)
             A, B = list(AB._nodes)
 
-            intersects = great_circle_arc.intersects_point(
+            intersects = gca.intersects_point(
                 A._point, B._point, nodes_array)
             intersection_indices = np.nonzero(intersects)[0]
 
             for index in intersection_indices:
                 node = nodes[index]
                 if node not in AB._nodes:
+                    changed = True
                     newA, newB = self._split_edge(AB, node)
 
                     new_edges = [
@@ -615,6 +632,7 @@ class Graph:
                             end_point._source_polygons)
                     edges = edges + new_edges
                     break
+        return changed
 
     def _find_arc_to_arc_intersections(self):
         # For speed, we want to vectorize all of the intersection
@@ -629,6 +647,7 @@ class Graph:
 
         # Calculate edge-to-edge intersections and break
         # edges on the intersection point.
+        changed = False
         while len(edges) > 1:
             AB = edges.pop(0)
             A = starts[0]; starts = starts[1:]  # numpy equiv of "pop(0)"
@@ -637,7 +656,7 @@ class Graph:
             # Calculate the intersection points between AB and all
             # other remaining edges
             with np.errstate(invalid='ignore'):
-                intersections = great_circle_arc.intersection(
+                intersections = gca.intersection(
                     A, B, starts, ends)
             # intersects is `True` everywhere intersections has an
             # actual intersection
@@ -649,6 +668,7 @@ class Graph:
             # we want to eliminate intersections that only intersect
             # at the end points
             for j in intersection_indices:
+                changed = True
                 CD = edges[j]
                 E = intersections[j]
 
@@ -683,6 +703,7 @@ class Graph:
                 ends = np.vstack(
                     (ends[:j], ends[j+1:], new_ends))
                 break
+        return changed
 
     def _find_all_intersections(self):
         """
@@ -690,8 +711,9 @@ class Graph:
         intersecting pair, four new edges are created around the
         intersection point.
         """
-        self._find_point_to_arc_intersections()
-        self._find_arc_to_arc_intersections()
+        changed = self._find_point_to_arc_intersections()
+        changed = self._find_arc_to_arc_intersections() or changed
+        return changed
 
     def _remove_interior_edges(self):
         """
@@ -711,7 +733,7 @@ class Graph:
                      (polygon in B._source_polygons or
                       polygon.contains_point(B._point))) and
                     polygon.contains_point(
-                        great_circle_arc.midpoint(A._point, B._point))):
+                        gca.midpoint(A._point, B._point))):
                     edge._count += 1
 
         for edge in list(self._edges):
@@ -741,7 +763,7 @@ class Graph:
                       (polygon in B._source_polygons or
                        polygon.contains_point(B._point)) and
                       polygon.contains_point(
-                          great_circle_arc.midpoint(A._point, B._point))):
+                          gca.midpoint(A._point, B._point))):
                     edge._count += 1
 
         for edge in list(self._edges):
@@ -807,13 +829,45 @@ class Graph:
                 break
         return changed
 
+    def _contains_inside_point(self, poly):
+        """
+        Check if the polygons in the graph all contain
+        the interior point of a polygon
+        """
+        for point in poly.inside:
+            for source_poly in self._source_polygons:
+                if not source_poly.contains_point(point):
+                    return False
+        return True
+
     def _trace_polygons(self):
         """
         Given a graph that has had cutlines removed and all
         intersections found, traces it to find a list of
         disjoint polygons
         """
-        from . import polygon as mpolygon
+
+        def edge_normal(edge, last_edge):
+            # THe normal vector to the plane defining the arc
+            normal = gca._cross_and_normalize(edge._nodes[0]._point,
+                                              edge._nodes[1]._point)
+            if last_edge is not None:
+                orientation = None
+                for i in (0, 1):
+                    last_edge_point = last_edge._nodes[i]._point
+                    for j in (0, 1):
+                        point = edge._nodes[j]._point
+                        if np.array_equal(last_edge_point, point):
+                            if i == j:
+                                orientation = -1.0
+                            else:
+                                orientation = 1.0
+
+                if orientation is None:
+                    raise RuntimeError("Unconnected edge found when tracing")
+                normal = orientation * normal
+
+            return normal
 
         def pick_next_edge(node, last_edge):
             # Pick the next edge when arriving at a node from
@@ -826,24 +880,22 @@ class Graph:
             for edge in node._edges:
                 if not edge._followed:
                     candidates.append(edge)
+
             if len(candidates) == 0:
                 raise ValueError("No more edges to follow")
-            elif len(candidates) == 1:
+            elif len(candidates) == 1 or last_edge is None:
                 return candidates[0]
 
-            for edge in candidates:
-                last_edge_cross = math_util.cross(
-                    last_edge._nodes[0]._point, last_edge._nodes[1]._point)
-                if (not
-                    np.all(math_util.cross(
-                        edge._nodes[0]._point, edge._nodes[1]._point) ==
-                           last_edge_cross) or
-                    np.all(math_util.cross(
-                        edge._nodes[0]._point, edge._nodes[1]._point) ==
-                           last_edge_cross)):
-                    return edge
+            last_edge_cross = edge_normal(last_edge, None)
+            edge_cross = [edge_normal(edge, last_edge) for edge in candidates]
+            edge_cross = np.asanyarray(edge_cross)
+            dot = gca.inner1d(edge_cross, last_edge_cross)
 
-            return candidates[0]
+            schwartz = zip(dot, candidates)
+            schwartz = sorted(schwartz, key=lambda x: x[0])
+
+            middle = len(candidates) // 2
+            return schwartz[middle][1]
 
         polygons = []
         edges = set(self._edges)  # copy
@@ -853,14 +905,15 @@ class Graph:
         while len(edges):
             points = []
             edge = edges.pop()
+            edge._followed = True
             start_node = node = edge._nodes[0]
+            points.append(node._point)
+            node = edge._nodes[1]
+            points.append(node._point)
             while True:
-                # TODO: Do we need this if clause any more?
-                if len(points):
-                    if not np.array_equal(points[-1], node._point):
-                        points.append(node._point)
-                else:
+                if not np.array_equal(points[-1], node._point):
                     points.append(node._point)
+
                 edge = pick_next_edge(node, edge)
                 edge._followed = True
                 edges.discard(edge)
@@ -880,5 +933,4 @@ class Graph:
         intersections found, traces it to find a resulting single
         polygon.
         """
-        from . import polygon as mpolygon
         return mpolygon.SphericalPolygon(self._trace_polygons())
