@@ -20,7 +20,6 @@ from . import vector
 
 __all__ = ['SphericalPolygon']
 
-
 class _SingleSphericalPolygon(object):
     r"""
     Polygons are represented by both a set of points (in Cartesian
@@ -30,13 +29,12 @@ class _SingleSphericalPolygon(object):
     therefore we need a way of specifying which is which.
     """
 
-    def __init__(self, points, inside=None, auto_close=False):
+    def __init__(self, points, inside=None):
         r"""
         Parameters
         ----------
         points : An Nx3 array of (*x*, *y*, *z*) triples in vector space
-            These points define the boundary of the polygon.  It must
-            be "closed", i.e., the last point is the same as the first.
+            These points define the boundary of the polygon.
 
             It may contain zero points, in which it defines the null
             polygon.  It may not contain one, two or three points.
@@ -47,9 +45,6 @@ class _SingleSphericalPolygon(object):
             This point must be inside the polygon.  If not provided, an
             interior point will be calculated
 
-        auto_close : bool, optional
-            If True, and the input polygon is not closed,
-            add the initial point to the polygon.
 
         """
         if len(points) == 0:
@@ -59,14 +54,12 @@ class _SingleSphericalPolygon(object):
             self._points = np.asanyarray(points)
             return
 
-        if auto_close:
+        if not np.array_equal(points[0], points[-1]):
             points = list(points[:])
             points.append(points[0])
 
         if len(points) < 3:
             raise ValueError("Polygon made of too few points")
-        else:
-            assert np.array_equal(points[0], points[-1]), 'Polygon is not closed'
 
         self._points = points = np.asanyarray(points)
         new_inside = self._find_new_inside()
@@ -82,8 +75,6 @@ class _SingleSphericalPolygon(object):
             if self.contains_point(new_inside) != self.is_clockwise():
                 self._points = points[::-1]
 
-        # TODO: Detect self-intersection and fix
-
     def __copy__(self):
         return deepcopy(self)
 
@@ -93,6 +84,15 @@ class _SingleSphericalPolygon(object):
         return '%s(%r, %r)' % (self.__class__.__name__,
                                self.points, self.inside)
 
+    def invert_polygon(self):
+        """
+        Compute the inverse (complement) of a single polygon
+        """
+        poly = self.copy()
+        poly._points = poly._points[::-1]
+        poly._inside = np.asanyarray(self._find_new_outside())
+        return poly
+
     @property
     def points(self):
         """
@@ -151,184 +151,13 @@ class _SingleSphericalPolygon(object):
         return vector.vector_to_lonlat(self.points[:,0], self.points[:,1],
                                       self.points[:,2], degrees=True)
 
-    @classmethod
-    def from_lonlat(cls, lon, lat, center=None, degrees=True,
-                    auto_close=False):
-        r"""
-        Create a new `SphericalPolygon` from a list of (*longitude*, *latitude*)
-        points.
-
-        Parameters
-        ----------
-        lon, lat : 1-D arrays of the same length
-            The vertices of the polygon in longitude and
-            latitude.  It must be \"closed\", i.e., that is, the
-            last point is the same as the first.
-
-        center : (*lon*, *lat*) pair, optional
-            A point inside of the polygon to define its inside.  If no
-            *center* point is provided, the mean of the polygon's
-            points in vector space will be used.  That approach may
-            not work for concave polygons.
-
-        degrees : bool, optional
-            If `True`, (default) *lon* and *lat* are in decimal degrees,
-            otherwise in radians.
-
-        auto_close : bool, optional
-            If True, input polygon is not closed, add the initial lon, lat pair
-            to the polygon.
-
-        Returns
-        -------
-        polygon : `SphericalPolygon` object
-        """
-        # Convert to Cartesian
-        x, y, z = vector.lonlat_to_vector(lon, lat, degrees=degrees)
-
-        points = np.dstack((x, y, z))[0]
-
-        if center is None:
-            center = points.mean(axis=0)
-            vector.normalize_vector(center, output=center)
-        else:
-            center = vector.lonlat_to_vector(*center, degrees=degrees)
-
-        return cls(points, center, auto_close=auto_close)
-
-    @classmethod
-    def from_cone(cls, lon, lat, radius, degrees=True, steps=16):
-        r"""
-        Create a new `_SingleSphericalPolygon` from a cone (otherwise known
-        as a "small circle") defined using (*lon*, *lat*, *radius*).
-
-        The cone is not represented as an ideal circle on the sphere,
-        but as a series of great circle arcs.  The resolution of this
-        conversion can be controlled using the *steps* parameter.
-
-        Parameters
-        ----------
-        lon, lat : float scalars
-            This defines the center of the cone
-
-        radius : float scalar
-            The radius of the cone
-
-        degrees : bool, optional
-            If `True`, (default) *lon*, *lat* and *radius* are in
-            decimal degrees, otherwise in radians.
-
-        steps : int, optional
-            The number of steps to use when converting the small
-            circle to a polygon.
-
-        Returns
-        -------
-        polygon : `_SingleSphericalPolygon` object
-        """
-        u, v, w = vector.lonlat_to_vector(lon, lat, degrees=degrees)
-        if degrees:
-            radius = np.deg2rad(radius)
-
-        # Get an arbitrary perpendicular vector.  This be be obtained
-        # by crossing (u, v, w) with any unit vector that is not itself.
-        which_min = np.argmin([u*u, v*v, w*w])
-        if which_min == 0:
-            perp = np.cross([u, v, w], [1., 0., 0.])
-        elif which_min == 1:
-            perp = np.cross([u, v, w], [0., 1., 0.])
-        else:
-            perp = np.cross([u, v, w], [0., 0., 1.])
-        perp = vector.normalize_vector(perp)
-
-        # Rotate by radius around the perpendicular vector to get the
-        # "pen"
-        x, y, z = vector.rotate_around(
-            u, v, w, perp[0], perp[1], perp[2], radius, degrees=False)
-
-        # Then rotate the pen around the center point all 360 degrees
-        C = np.linspace(0, np.pi * 2.0, steps)
-        # Ensure that the first and last elements are exactly the
-        # same.  2π should equal 0, but with rounding error that isn't
-        # always the case.
-        C[-1] = 0
-        C = C[::-1]
-        X, Y, Z = vector.rotate_around(x, y, z, u, v, w, C, degrees=False)
-
-        return cls(np.dstack((X, Y, Z))[0], (u, v, w))
-
-    @classmethod
-    def from_wcs(cls, fitspath, steps=1, crval=None):
-        r"""
-        Create a new `_SingleSphericalPolygon` from the footprint of a FITS
-        WCS specification.
-
-        This method requires having `astropy <http://astropy.org>`__
-        installed.
-
-        Parameters
-        ----------
-        fitspath : path to a FITS file, `astropy.io.fits.Header`, or `astropy.wcs.WCS`
-            Refers to a FITS header containing a WCS specification.
-
-        steps : int, optional
-            The number of steps along each edge to convert into
-            polygon edges.
-
-        Returns
-        -------
-        polygon : `_SingleSphericalPolygon` object
-        """
-        from astropy import wcs as pywcs
-        from astropy.io import fits
-
-        if isinstance(fitspath, fits.Header):
-            header = fitspath
-            wcs = pywcs.WCS(header)
-        elif isinstance(fitspath, pywcs.WCS):
-            wcs = fitspath
-        else:
-            wcs = pywcs.WCS(fitspath)
-        if crval is not None:
-            wcs.wcs.crval = crval
-        xa, ya = [wcs._naxis1, wcs._naxis2]
-
-        length = steps * 4 + 1
-        X = np.empty(length)
-        Y = np.empty(length)
-
-        # Now define each of the 4 edges of the quadrilateral
-        X[0      :steps  ] = np.linspace(1, xa, steps, False)
-        Y[0      :steps  ] = 1
-        X[steps  :steps*2] = xa
-        Y[steps  :steps*2] = np.linspace(1, ya, steps, False)
-        X[steps*2:steps*3] = np.linspace(xa, 1, steps, False)
-        Y[steps*2:steps*3] = ya
-        X[steps*3:steps*4] = 1
-        Y[steps*3:steps*4] = np.linspace(ya, 1, steps, False)
-        X[-1]              = 1
-        Y[-1]              = 1
-
-        # Use wcslib to convert to (lon, lat)
-        lon, lat = wcs.all_pix2world(X, Y, 1)
-
-        # Convert to Cartesian
-        x, y, z = vector.lonlat_to_vector(lon, lat)
-
-        # Calculate an inside point
-        lon, lat = wcs.all_pix2world(xa / 2.0, ya / 2.0, 1)
-        xc, yc, zc = vector.lonlat_to_vector(lon, lat)
-
-        return cls(np.dstack((x, y, z))[0], (xc, yc, zc))
-
     def _contains_point(self, point, P, r):
         point = np.asanyarray(point)
-        if great_circle_arc.same_point(r, point):
+        if np.array_equal(r, point):
             return True
 
         intersects = great_circle_arc.intersects(P[:-1], P[1:], r, point)
         crossings = np.sum(intersects)
-
         return (crossings % 2) == 0
 
     def contains_point(self, point):
@@ -473,12 +302,10 @@ class _SingleSphericalPolygon(object):
         if len(self._points) < 3:
             return np.array(0.0)
 
-        points = self._points
-        angles = np.hstack([
-            great_circle_arc.angle(
-                points[:-2], points[1:-1], points[2:], degrees=False),
-            great_circle_arc.angle(
-                points[-2], points[0], points[1], degrees=False)])
+        points = np.vstack((self._points, self._points[1]))
+        angles = great_circle_arc.angle(points[:-2], points[1:-1],
+                                        points[2:], degrees=False)
+
         return np.sum(angles) - (len(angles) - 2) * np.pi
 
     def union(self, other):
@@ -516,8 +343,7 @@ class _SingleSphericalPolygon(object):
     def _find_new_inside(self):
         """
         Finds an acceptable inside point inside of *points* that is
-        also inside of *polygons*.  Used by the intersection
-        algorithm and the area computation.
+        also inside of *polygons*.
         """
         npoints = len(self._points)
         if npoints > 4:
@@ -537,6 +363,25 @@ class _SingleSphericalPolygon(object):
             vector.normalize_vector(inside, output=inside)
         return inside
 
+    def _find_new_outside(self):
+        """
+        Finds an acceptable point outside of the polygon
+        """
+        tagged_points = []
+        points = self._points[:-1]
+
+        # Compute the minimum distance between all polygon points
+        # and each antipode to a polygon point
+        for point in points:
+            point = -1.0 * point
+            dot = great_circle_arc.inner1d(point, points)
+            tag = np.amax(dot)
+            tagged_points.append((tag, point))
+
+        # find the antipode with the maximum distance
+        # to any polygon point. It is our outside point.
+        (tag, point) = min(tagged_points, key=lambda p: p[0])
+        return point
 
     def intersection(self, other):
         """
@@ -644,7 +489,7 @@ class SphericalPolygon(object):
     This class contains a list of disjoint closed polygons.
     """
 
-    def __init__(self, init, inside=None, auto_close=False):
+    def __init__(self, init, inside=None):
         r"""
         Parameters
         ----------
@@ -654,23 +499,14 @@ class SphericalPolygon(object):
 
                - An Nx3 array of (*x*, *y*, *z*) triples in Cartesian
                  space.  These points define the boundary of the
-                 polygon.  It must be "closed", i.e., the last point
-                 is the same as the first.
+                 polygon.
 
                  It may contain zero points, in which it defines the
-                 null polygon.  It may not contain one, two or three
-                 points.  Four points are needed to define a triangle,
-                 since the polygon must be closed.
+                 null polygon.  It may not contain one or two points.
 
         inside : An (*x*, *y*, *z*) triple, optional
             If *init* is an array of points, this point must be inside
-            the polygon.  If not provided, the mean of the points will
-            be used.
-
-        auto_close : bool, optional
-            If True, input polygon is not closed, add the initial point
-            to the polygon.
-
+            the polygon.  If it is not provided, one will be created.
 
         """
         from . import graph
@@ -681,8 +517,7 @@ class SphericalPolygon(object):
             self._polygons = tuple(init)
             return
 
-        self._polygons = (_SingleSphericalPolygon(init, inside,
-                                                  auto_close=auto_close),)
+        self._polygons = (_SingleSphericalPolygon(init, inside),)
 
         polygons = []
         for polygon in self.iter_polygons_flat():
@@ -696,6 +531,9 @@ class SphericalPolygon(object):
 
     copy = __copy__
 
+    def __len__(self):
+        return len(self._polygons)
+
     def iter_polygons_flat(self):
         """
         Iterate over all base polygons that make up this multi-polygon
@@ -752,6 +590,17 @@ class SphericalPolygon(object):
             cw = None
         return cw
 
+    @staticmethod
+    def self_intersect(points):
+        """
+        Return true if the path defined by a list of points
+        intersects itself
+        """
+        from . import graph
+        polygon = _SingleSphericalPolygon(points)
+        g = graph.Graph((polygon,))
+        return g._find_all_intersections()
+
     def to_lonlat(self):
         """
         Convert the `SphericalPolygon` footprint to longitude and latitude
@@ -766,98 +615,49 @@ class SphericalPolygon(object):
         for polygon in self.iter_polygons_flat():
             yield polygon.to_lonlat()
 
-    def to_radec(self):
-        """
-        Convert the `SphericalPolygon` footprint to right ascension and
-        declination coordinates.
-
-        Returns
-        -------
-        polyons : iterator
-            Each element in the iterator is a tuple of the form (*lon*,
-            *lat*), where each is an array of points.
-        """
-        yield self.to_lonlat()
+    # to_ra_dec is an alias for to_lonlat
+    to_radec = to_lonlat
 
     @classmethod
-    def from_lonlat(cls, lon, lat, center=None, degrees=True,
-                    auto_close=False):
+    def from_lonlat(cls, lon, lat, center=None, degrees=True):
         r"""
-        Create a new `SphericalPolygon` from a list of (*lon*, *lat*)
+        Create a new `SphericalPolygon` from a list of (*longitude*, *latitude*)
         points.
 
         Parameters
         ----------
         lon, lat : 1-D arrays of the same length
             The vertices of the polygon in longitude and
-            latitude.  It must be \"closed\", i.e., that is, the
-            last point is the same as the first.
+            latitude.
 
         center : (*lon*, *lat*) pair, optional
-            A point inside of the polygon to define its inside.  If no
-            *center* point is provided, the mean of the polygon's
-            points in vector space will be used.  That approach may
-            not work for concave polygons.
+            A point inside of the polygon to define its inside.
 
         degrees : bool, optional
             If `True`, (default) *lon* and *lat* are in decimal degrees,
             otherwise in radians.
 
-        auto_close : bool, optional
-            If True, input polygon is not closed, add the initial ;pn, lat pair
-            to the polygon.
-
-
         Returns
         -------
         polygon : `SphericalPolygon` object
         """
-        return cls([
-            _SingleSphericalPolygon.from_lonlat(
-                lon, lat, center=center, degrees=degrees,
-                auto_close=auto_close)])
+        # Convert to Cartesian
+        x, y, z = vector.lonlat_to_vector(lon, lat, degrees=degrees)
 
-    @classmethod
-    def from_radec(cls, ra, dec, center=None, degrees=True,
-                   auto_close=False):
-        r"""
-        Create a new `SphericalPolygon` from a list of (*ra*, *dec*)
-        points.
+        points = np.dstack((x, y, z))[0]
 
-        Parameters
-        ----------
-        ra, dec : 1-D arrays of the same length
-            The vertices of the polygon in right ascension and
-            declination.  It must be \"closed\", i.e., that is, the
-            last point is the same as the first.
+        if center is not None:
+            center = vector.lonlat_to_vector(*center, degrees=degrees)
 
-        center : (*ra*, *dec*) pair, optional
-            A point inside of the polygon to define its inside.  If no
-            *center* point is provided, the mean of the polygon's
-            points in vector space will be used.  That approach may
-            not work for concave polygons.
+        return cls(points, center)
 
-        degrees : bool, optional
-            If `True`, (default) *ra* and *dec* are in decimal degrees,
-            otherwise in radians.
-
-        auto_close : bool, optional
-            If True, input polygon is not closed, add the initial ra, dec pair
-            to the polygon.
-
-        Returns
-        -------
-        polygon : `SphericalPolygon` object
-        """
-        return cls([
-            _SingleSphericalPolygon.from_lonlat(
-                ra, dec, center=center, degrees=degrees,
-                auto_close=auto_close)])
+    # from_radec is an alias for from_lon_lat
+    from_radec = from_lonlat
 
     @classmethod
     def from_cone(cls, lon, lat, radius, degrees=True, steps=16):
         r"""
-        Create a new `SphericalPolygon` from a cone (otherwise known
+        Create a new `_SingleSphericalPolygon` from a cone (otherwise known
         as a "small circle") defined using (*lon*, *lat*, *radius*).
 
         The cone is not represented as an ideal circle on the sphere,
@@ -884,14 +684,41 @@ class SphericalPolygon(object):
         -------
         polygon : `SphericalPolygon` object
         """
-        return cls([
-            _SingleSphericalPolygon.from_cone(
-                lon, lat, radius, degrees=degrees, steps=steps)])
+        u, v, w = vector.lonlat_to_vector(lon, lat, degrees=degrees)
+        if degrees:
+            radius = np.deg2rad(radius)
+
+        # Get an arbitrary perpendicular vector.  This be be obtained
+        # by crossing (u, v, w) with any unit vector that is not itself.
+        which_min = np.argmin([u*u, v*v, w*w])
+        if which_min == 0:
+            perp = np.cross([u, v, w], [1., 0., 0.])
+        elif which_min == 1:
+            perp = np.cross([u, v, w], [0., 1., 0.])
+        else:
+            perp = np.cross([u, v, w], [0., 0., 1.])
+        perp = vector.normalize_vector(perp)
+
+        # Rotate by radius around the perpendicular vector to get the
+        # "pen"
+        x, y, z = vector.rotate_around(
+            u, v, w, perp[0], perp[1], perp[2], radius, degrees=False)
+
+        # Then rotate the pen around the center point all 360 degrees
+        C = np.linspace(0, np.pi * 2.0, steps)
+        # Ensure that the first and last elements are exactly the
+        # same.  2π should equal 0, but with rounding error that isn't
+        # always the case.
+        C[-1] = 0
+        C = C[::-1]
+        X, Y, Z = vector.rotate_around(x, y, z, u, v, w, C, degrees=False)
+
+        return cls(np.dstack((X, Y, Z))[0], (u, v, w))
 
     @classmethod
     def from_wcs(cls, fitspath, steps=1, crval=None):
         r"""
-        Create a new `SphericalPolygon` from the footprint of a FITS
+        Create a new `_SingleSphericalPolygon` from the footprint of a FITS
         WCS specification.
 
         This method requires having `astropy <http://astropy.org>`__
@@ -910,9 +737,59 @@ class SphericalPolygon(object):
         -------
         polygon : `SphericalPolygon` object
         """
-        return cls([
-            _SingleSphericalPolygon.from_wcs(
-                fitspath, steps=steps, crval=crval)])
+        from astropy import wcs as pywcs
+        from astropy.io import fits
+
+        if isinstance(fitspath, fits.Header):
+            header = fitspath
+            wcs = pywcs.WCS(header)
+        elif isinstance(fitspath, pywcs.WCS):
+            wcs = fitspath
+        else:
+            wcs = pywcs.WCS(fitspath)
+        if crval is not None:
+            wcs.wcs.crval = crval
+        xa, ya = [wcs._naxis1, wcs._naxis2]
+
+        length = steps * 4 + 1
+        X = np.empty(length)
+        Y = np.empty(length)
+
+        # Now define each of the 4 edges of the quadrilateral
+        X[0      :steps  ] = np.linspace(1, xa, steps, False)
+        Y[0      :steps  ] = 1
+        X[steps  :steps*2] = xa
+        Y[steps  :steps*2] = np.linspace(1, ya, steps, False)
+        X[steps*2:steps*3] = np.linspace(xa, 1, steps, False)
+        Y[steps*2:steps*3] = ya
+        X[steps*3:steps*4] = 1
+        Y[steps*3:steps*4] = np.linspace(ya, 1, steps, False)
+        X[-1]              = 1
+        Y[-1]              = 1
+
+        # Use wcslib to convert to (lon, lat)
+        lon, lat = wcs.all_pix2world(X, Y, 1)
+
+        # Convert to Cartesian
+        x, y, z = vector.lonlat_to_vector(lon, lat)
+
+        # Calculate an inside point
+        lon, lat = wcs.all_pix2world(xa / 2.0, ya / 2.0, 1)
+        xc, yc, zc = vector.lonlat_to_vector(lon, lat)
+
+        return cls(np.dstack((x, y, z))[0], (xc, yc, zc))
+
+    def invert_polygon(self):
+        """
+        Construct a polygon which is the inverse (complement) of the
+        original polygon
+        """
+        if len(self._polygons) != 1:
+            raise RuntimeErrror("Can only invert a single polygon")
+
+        klass = self.__class__
+        inverted_polygon = self._polygons[0].invert_polygon()
+        return klass((inverted_polygon, ))
 
     def contains_point(self, point):
         r"""
